package org.jboss.support.signals;

import org.apache.log4j.Logger;
import org.jboss.system.ServiceMBeanSupport;

import sun.misc.Signal;
import sun.misc.SignalHandler;

/**
 * 
 * @author darran.lofthouse@jboss.com
 */
public class SignalService extends ServiceMBeanSupport implements SignalServiceMBean, SignalHandler
{

   private static final Logger log = Logger.getLogger(SignalService.class);

   private String signals = null;

   public String getSignals()
   {
      return signals;
   }

   public void setSignals(final String signals)
   {
      if (this.signals != null)
      {
         throw new IllegalStateException("setSignals should not be called after it has first been set.");
      }

      this.signals = signals;
   }

   public void addSignal(String signal)
   {
      if (this.signals != null && this.signals.contains(signal))
      {
         log.warn("Signal '" + signal + "' is already registered");

         return;
      }

      addHandler(signal);
      if (signals == null || signals.length() == 0)
      {
         signals = signal;
      }
      else
      {
         signals = signals + "," + signal;
      }

   }

   // Method from SignalHanlder interface.

   protected void startService() throws Exception
   {
      log.info("Start");

      if (signals == null || signals.length() == 0)
      {
         log.info("No signals set, exiting");
         return;
      }

      String[] s = signals.split(",");

      for (int i = 0; i < s.length; i++)
      {
         String currentSignal = s[i];

         addHandler(currentSignal);
      }

   }

   private void addHandler(final String signalName)
   {
      Signal signal = new Signal(signalName);
      Signal.handle(signal, this);
      log.info("Signal handler registered for signal '" + signalName + "'");
   }

   public void handle(Signal signal)
   {
      log.warn("** Signal received Number=" + signal.getNumber() + " Name=" + signal.getName()
            + " Signal has been ignored!");
   }

}
